use embassy_rp::{watchdog::Watchdog, peripherals};
use embassy_time::{Duration, Timer};


// The pause bits are on by default after reset but I think Watchdog::enable()
// disables them so let's enable them, again.
fn watchdog_pause_on_debug(_watchdog: &mut Watchdog) {
    // This turns off the watchdog - no good! :-/
    //_watchdog.pause_on_debug(true);

    use embassy_rp::pac;
    let watchdog = pac::WATCHDOG;
    //SAFETY: We know which values have been set by WatchdogFlash / Watchdog.
    unsafe {
        // We use modify() to keep the enable bit at its current value.
        watchdog.ctrl().modify(|w| {
            w.set_pause_dbg0(true);
            w.set_pause_dbg1(true);
            w.set_pause_jtag(true);
        });
    }
}

pub struct WatchdogFixed {
    inner: Watchdog,
}

impl WatchdogFixed {
    pub fn new(watchdog: peripherals::WATCHDOG) -> Self {
        WatchdogFixed {
            inner: Watchdog::new(watchdog)
        }
    }

    pub fn start(self: &mut Self) {
        // Use the maximum value and turn on "pause on debug" to not interfere with
        // probe-rs-cli's flash programming (which doesn't seem to turn off or feed
        // the watchdog). This seems to work well enough but if it doesn't, manually
        // put the board into bootloader mode before programming flash.
        self.inner.start(Duration::from_secs(8));
        watchdog_pause_on_debug(&mut self.inner);    
    }

    pub fn feed(self: &mut Self) {
        self.inner.feed();
    }

    pub async fn start_and_feed_continuously(self: &mut Self) {
        self.start();
        loop {
            self.feed();
            Timer::after(Duration::from_secs(2)).await;
        }
    }
}
